import { defineComponent, onMounted, reactive, ref } from 'vue';
import styles from './index.module.less';
import CardType from '/src/components/card-type';
import Pagination from '/src/components/pagination';
import SearchGroupResources from './search-group-resources';
import { favorite, materialQueryPage } from '../../api';
import { NModal, NSpin } from 'naive-ui';
import TheEmpty from '/src/components/TheEmpty';
import CardPreview from '/src/components/card-preview';
import AddTeaching from '../../model/add-teaching';
// import ShareResourcesGuide from '@/custom-plugins/guide-page/shareResources-guide';
import { modalClickMask } from '/src/state';

export default defineComponent({
  name: 'share-resources',
  setup() {
    const state = reactive({
      searchWord: '',
      loading: false,
      pageTotal: 0,
      pagination: {
        page: 1,
        rows: 20
      },
      searchGroup: {
        type: 'MUSIC', //
        name: '',
        audioPlayTypes: [] as any,
        musicalInstrumentId: null,
        bookVersionId: null,
        subjectId: null,
        sourceType: 2
      },
      tableList: [] as any,
      teachingStatus: false,
      show: false,
      item: {} as any
    });
    // const showGuide = ref(false);
    const SearchGroupResourcesRef = ref();
    const getList = async () => {
      try {
        state.loading = true;
        const { data } = await materialQueryPage({
          ...state.searchGroup,
          ...state.pagination
        });
        state.loading = false;
        state.pageTotal = Number(data.total);
        const tempRows = data.rows || [];
        const temp: any = [];
        tempRows.forEach((row: any) => {
          temp.push({
            id: row.id,
            coverImg: row.coverImg,
            type: row.type,
            title: row.name,
            isCollect: !!row.favoriteFlag,
            isSelected: row.sourceFrom === 'PLATFORM' ? true : false,
            refFlag: row.refFlag,
            audioPlayTypeArray: row.audioPlayTypes
              ? row.audioPlayTypes.split(',')
              : [],
            background: row.background,
            content: row.content
          });
        });
        state.tableList = temp || [];
        // setTimeout(() => {
        //   showGuide.value = true;
        // }, 500);
      } catch {
        state.loading = false;
      }
    };

    const onSearch = async (item: any) => {
      state.pagination.page = 1;

      const { subjectId, ...res } = item;
      state.searchGroup = Object.assign(state.searchGroup, {
        ...res,
        musicalInstrumentId: subjectId,
        subjectId: null
      });

      getList();
    };

    // 收藏
    const onCollect = async (item: any) => {
      try {
        await favorite({
          materialId: item.id,
          favoriteFlag: item.isCollect ? 0 : 1,
          type: item.type
        });
        item.isCollect = !item.isCollect;
      } catch {
        //
      }
    };

    onMounted(() => {
      // 会在搜索组件里面调用，因为有初始化搜索条件
      // getList();
    });
    return () => (
      <>
        <SearchGroupResources
          ref={SearchGroupResourcesRef}
          onSearch={(item: any) => onSearch(item)}
          onAdd={() => (state.teachingStatus = true)}
        />
        <NSpin v-model:show={state.loading} style={{ 'min-height': '50vh' }}>
          <div class={styles.list}>
            {state.tableList.map((item: any, index: number) => (
              <div class={styles.itemWrap}>
                <div class={styles.itemWrapBox}>
                  {index == 0 ? (
                    <CardType
                      {...{ id: 'shareResources-1' }}
                      item={item}
                      disabledMouseHover={false}
                      onClick={(val: any) => {
                        if (val.type === 'IMG') return;
                        state.show = true;
                        state.item = {
                          instrumentId:
                            state.searchGroup.musicalInstrumentId || null,
                          ...val
                        };
                      }}
                      onCollect={(item: any) => onCollect(item)}
                    />
                  ) : (
                    <CardType
                      item={item}
                      disabledMouseHover={false}
                      onClick={(val: any) => {
                        if (val.type === 'IMG') return;
                        state.show = true;
                        state.item = {
                          instrumentId:
                            state.searchGroup.musicalInstrumentId || null,
                          ...val
                        };
                      }}
                      onCollect={(item: any) => onCollect(item)}
                    />
                  )}
                </div>
              </div>
            ))}

            {!state.loading && state.tableList.length <= 0 && (
              <TheEmpty
                style={{ minHeight: '50vh' }}
                description="暂无共享资源"
              />
            )}
          </div>
        </NSpin>

        <Pagination
          v-model:page={state.pagination.page}
          v-model:pageSize={state.pagination.rows}
          v-model:pageTotal={state.pageTotal}
          onList={getList}
        />

        {/* 弹窗查看 */}
        <CardPreview
          v-model:show={state.show}
          item={state.item}
          isDownload={false}
        />

        {/* 添加自定义教材 */}
        <NModal
          maskClosable={modalClickMask}
          v-model:show={state.teachingStatus}
          preset="card"
          showIcon={false}
          class={['modalTitle background', styles.teachingModal]}
          title={'自定义教材'}
          blockScroll={false}>
          <AddTeaching onClose={() => (state.teachingStatus = false)} />
        </NModal>
        {/* {showGuide.value ? <ShareResourcesGuide></ShareResourcesGuide> : null} */}
      </>
    );
  }
});
